from abc import ABC, abstractmethod
from typing import Union


class Abstract(ABC):

    def __init__(
            self,
            year: Union[int, str],
            month: Union[int, str],
            day: Union[int, str]
    ):
        try:
            self.year = int(year)
            self.month = int(month)
            self.day = int(day)
        except ValueError:
            raise ValueError('allowed data is numeric')

    @classmethod
    @abstractmethod
    def now(cls):
        ...

    @abstractmethod
    def weekday(self):
        ...

    @abstractmethod
    def elapsedtime(self):
        ...


class HijriAbstract(Abstract):

    def __init__(self, year, month, day):
        super().__init__(year, month, day)

    @abstractmethod
    def hijri_to_gregorian(self):
        pass

    @abstractmethod
    def hijri_to_jalali(self):
        pass


class GregorianAbstract(Abstract):

    def __init__(self, year, month, day):
        super().__init__(year, month, day)

    @abstractmethod
    def gregorian_to_hijri(self):
        pass

    @abstractmethod
    def gregorian_to_jalali(self):
        pass


class JalaliAbstract(Abstract):

    def __init__(self, year, month, day):
        super().__init__(year, month, day)

    @abstractmethod
    def jalali_to_hijri(self):
        pass

    @abstractmethod
    def jalali_to_gregorian(self):
        pass



from ..core import JalaliAbstract
from persiantools.jdatetime import JalaliDate, JalaliDateTime
from hijridate import Gregorian
from typing import Tuple
import pytz
from datetime import date


class JalaliConverter(JalaliAbstract):

    def __init__(
            self,
            year=None,
            month=None,
            day=None
    ) -> None:
        if not year or not month or not day:
            raise TypeError("missing 3 required positional arguments: 'year', 'month', and 'day'")
        super().__init__(year, month, day)

    def jalali_to_hijri(self) -> Tuple:
        try:
            year, month, day = self.jalali_to_gregorian()
            gregorian_date = Gregorian(year, month, day)
            hijri_date = gregorian_date.to_hijri()
            return hijri_date.year, hijri_date.month, hijri_date.day
        except ValueError:
            raise ValueError

    def jalali_to_gregorian(self) -> Tuple:
        try:
            time = JalaliDate(self.year, self.month, self.day).to_gregorian()
            return time.year, time.month, time.day
        except ValueError:
            raise ValueError

    @classmethod
    def now(cls) -> Tuple:
        now = JalaliDate.today()
        return now.year, now.month, now.day

    def weekday(self) -> str:
        try:
            time = JalaliDateTime(self.year, self.month, self.day, tzinfo=pytz.utc).strftime("%c")
            return time.split(' ')[0]
        except ValueError:
            raise ValueError

    def elapsedtime(self) -> Tuple:
        try:
            date1 = JalaliDate(self.year, self.month, self.day).to_gregorian()
            now = JalaliDate(self.year, self.month, self.day).today().to_gregorian()
            date2 = date(now.year, now.month, now.day)
            delta = date2 - date1
            days = delta.days

            # Calculate the difference in years and months
            years = days // 365
            months = (days % 365) // 30  # Approximation: assumes a month is  30 days

            # Calculate the remaining days after accounting for years and months
            remaining_days = (days % 365) % 30

            return years, months, remaining_days
        except ValueError:
            raise ValueError



from setuptools import setup, find_packages

# Read the README.md file
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


def read_version():
    version = {}
    with open("nonadateconverter/version.py", "r") as f:
        exec(f.read(), version)
        return version["__version__"]


setup(
    name="nonadateconverter",
    version=read_version(),
    author="Navid Nourazar",
    author_email="navid.nourazar@gmail.com",
    description="dateconverter package for Gregorian, Hijri and Jalali",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/naweed1992/nonadateconverter",
    packages=find_packages(
        include=["nonadateconverter", "nonadateconverter.*"]
    ),  # would be the same as name
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=open(
        "requirements.txt"
    ).readlines(),  # external packages acting as dependencies
)



from ..core import GregorianAbstract
from hijridate import Gregorian
import jdatetime
from typing import Tuple
from datetime import datetime, timezone, timedelta


class GregorianConverter(GregorianAbstract):
    def __init__(
            self,
            year=None,
            month=None,
            day=None
    ) -> None:
        if not year or not month or not day:
            raise TypeError("missing 3 required positional arguments: 'year', 'month', and 'day'")
        super().__init__(year, month, day)

    def gregorian_to_hijri(self) -> Tuple:
        try:
            gregorian_date = Gregorian(self.year, self.month, self.day)
            hijri_date = gregorian_date.to_hijri()
            return hijri_date.year, hijri_date.month, hijri_date.day
        except ValueError:
            raise ValueError

    def gregorian_to_jalali(self):
        try:
            gregorian_date = jdatetime.date.fromgregorian(year=self.year, month=self.month, day=self.day)
            return gregorian_date.year, gregorian_date.month, gregorian_date.day
        except ValueError:
            raise ValueError

    @classmethod
    def now(cls) -> Tuple:
        now = datetime.now()
        return now.year, now.month, now.day

    def weekday(self) -> str:
        try:
            timezone_local = timezone(timedelta(hours=3, minutes=30))
            date = datetime.now(tz=timezone_local)
            weekday = date.strftime("%A")
            return weekday
        except ValueError:
            raise ValueError

    def elapsedtime(self) -> Tuple:
        try:
            date1 = datetime(self.year, self.month, self.day)
            now = datetime.now()
            date2 = datetime(now.year, now.month, now.day)
            delta = date2 - date1
            days = delta.days

            # Calculate the difference in years and months
            years = days // 365
            months = (days % 365) // 30  # Approximation: assumes a month is  30 days

            # Calculate the remaining days after accounting for years and months
            remaining_days = (days % 365) % 30

            return years, months, remaining_days
        except ValueError:
            raise ValueError



from ..core import HijriAbstract
from hijridate import Hijri, Gregorian
from persiantools.jdatetime import JalaliDate
from typing import Tuple
from datetime import datetime, date


class HijriConverter(HijriAbstract):

    def __init__(
            self,
            year=None,
            month=None,
            day=None
    ) -> None:
        if not year or not month or not day:
            raise TypeError("missing 3 required positional arguments: 'year', 'month', and 'day'")
        super().__init__(year, month, day)

    def hijri_to_gregorian(self) -> Tuple:
        try:
            hijri_date = Hijri(self.year, self.month, self.day).to_gregorian()
            return hijri_date.year, hijri_date.month, hijri_date.day
        except ValueError:
            raise ValueError

    def hijri_to_jalali(self) -> Tuple:
        try:
            gregorian_date = self.hijri_to_gregorian()
            year, month, day = gregorian_date
            jalali_date = JalaliDate.to_jalali(year, month, day)
            return jalali_date.year, jalali_date.month, jalali_date.day
        except ValueError:
            raise ValueError

    def weekday(self) -> str:
        try:
            hijri_date = Hijri(self.year, self.month, self.day).to_gregorian()

            # Create a datetime object from the Gregorian date
            date_object = datetime(hijri_date.year, hijri_date.month, hijri_date.day)

            # Get the weekday of the date
            weekday = date_object.weekday()

            # Define a list of weekdays
            weekdays = ["Al Ithnayn", "Ath Thulatha", "Al Arbia", "Al khamis", "Al Jumuah", "As Sabt", "Al Ahad"]

            return weekdays[weekday]
        except ValueError:
            raise ValueError

    def elapsedtime(self) -> Tuple:
        try:
            date1 = datetime(self.year, self.month, self.day)
            now = Hijri(self.year, self.month, self.day).today()
            date2 = datetime(now.year, now.month, now.day)
            delta = date2 - date1
            days = delta.days

            # Calculate the difference in years and months
            years = days // 365
            months = (days % 365) // 30  # Approximation: assumes a month is  30 days

            # Calculate the remaining days after accounting for years and months
            remaining_days = (days % 365) % 30

            return years, months, remaining_days
        except ValueError:
            raise ValueError

    @classmethod
    def now(cls) -> Tuple:
        # Get today's date
        today = date.today()

        # Convert today's Gregorian date to Hijri
        hijri_today = Gregorian(today.year, today.month, today.day).to_hijri()
        return hijri_today.year, hijri_today.month, hijri_today.day
